/* 
 * $TSUKUBA_Release: Omni OpenMP/XcalableMP Compiler Software Version 0.6.0 (alpha) $
 * $TSUKUBA_Copyright:
 *  Copyright (C) 2010-2011 University of Tsukuba, 
 *  	      2012  University of Tsukuba and Riken AICS
 *  
 *  This software is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License version
 *  2.1 published by the Free Software Foundation.
 *  
 *  Please check the Copyright and License information in the files named
 *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
 *  Software release kit.
 *  
 *  * The specification of XcalableMP has been designed by the XcalableMP
 *    Specification Working Group (http://www.xcalablemp.org/).
 *  
 *  * The development of this software was partially supported by "Seamless and
 *    Highly-productive Parallel Programming Environment for
 *    High-performance computing" project funded by Ministry of Education,
 *    Culture, Sports, Science and Technology, Japan.
 *  $
 */
package xcodeml.c.util;

import xcodeml.util.*;

import org.w3c.dom.*;
import javax.xml.xpath.*;
import java.util.Set;
import java.util.List;
import java.util.ArrayList;

/**
 * Collector of variable references.
 */
public class XmcVarCollector {
    public XmcVarCollector(Set<String> variables) {
        this.variables = variables;

        XPathFactory xpathFactory = XPathFactory.newInstance();
        XPath xpath = xpathFactory.newXPath();

        String[] xpathExprStrings = {
            "descendant::Var/text()",
            "descendant::varAddr/text()",
            "descendant::funcAddr/text()",
            "descendant::memberRef/text()",
        };

        xpathExprs = new ArrayList<XPathExpression>();
        try {
            for (String xpathExprStr : xpathExprStrings) {
                xpathExprs.add(xpath.compile(xpathExprStr));
            }
        } catch (XPathExpressionException e) {
            throw new XmTranslationException(e);
        }
    }

    public void collectVars(Node node) {
        try {
            for (XPathExpression expr : xpathExprs) {
                Object result = expr.evaluate(node, XPathConstants.NODESET);
                NodeList nodes = (NodeList) result;
                for (int i = 0; i < nodes.getLength(); i++) {
                    variables.add(nodes.item(i).getNodeValue());
                }
            }
        } catch (XPathExpressionException e) {
            throw new XmTranslationException(node, e);
        }
    }

    private Set<String> variables;
    private List<XPathExpression> xpathExprs;
}
